==> sli-plots-0.0.26.tgz: package/addon/components/plot-axes-and-ticks.js <==
import Component from '@glimmer/component';
import { scaleTime, scaleLinear } from "d3-scale";
import { format } from 'date-fns'

export default class PlotAxesAndTicksComponent extends Component {
  get displayingXTicksOrLabels() {
    return this.args.showXTicks || !this.args.hideXTickLabels;
  }

  get dateLabelFormat() {
    return this.args.dateLabelFormat || "EEE";
  }

  get xScaleRange() {
    return this.args.dimensions
    ? scaleTime().range([0, this.args.dimensions.width])
    : {}
  }

  get yScaleRange() {
    return this.args.dimensions
    ? scaleLinear().range([this.args.dimensions.height, 0])
      : {}
  }

  makeYTicks(originalMin, max) {
    const { isMultiDataset, data, hasData, dimensions, showEveryTick } = this.args;
    const min = this.roundMinimum(originalMin, max);
    if((isMultiDataset && data[0].length === 0) || !hasData) {
      return { yScale: this.yScaleRange.domain ? this.yScaleRange.domain([min, max]) : () => {} }
    }

    const tickCount = dimensions.height < 500 ? 7 : 10;
    const stepSize = this.tickStepSize(max - min, tickCount);
    const maxTick = Math.ceil(max / stepSize) * stepSize;
    const yScale = this.yScaleRange.domain([min, maxTick]);
    return {
      max: maxTick,
      min,
      yScale,
      ticks: Array.from({length: showEveryTick ? 2 : Math.ceil((maxTick - min)/(stepSize + 1))}, (_, i) => {
        const value = showEveryTick
                        ? i === 0 ? min : maxTick
                        : i * stepSize + min
        const fixedValue = value < 1 && value !== 0 ? value.toFixed(2) : value;
        const y = yScale(fixedValue);
        return {
          label: `${fixedValue}`,
          fixedValue,
          y,
          position: `top: ${y}px;`
        }
      })
    }
  }

  // This method returns a new minimum value that is rounded down based on
  // the magnatude of of min max for a nice rounded value that isn't too far off
  // from the lowest value
  roundMinimum(min, max) {
    if(min === 0) return min;

    const diff = max - min;

    if(diff < 20) return Math.floor(min);
    if(diff < 100) return Math.floor(min / 10) * 10;
    if(diff < 2000) return Math.floor(min / 100) * 100;

    return Math.floor(min / 1000) * 1000;
  }

  get yScaling() {
    if(!this.args.data) return [0,0]
    const dataMax = this.getYMax();
    const dataMin = this.getYMin();
    const yTicksInfo = this.makeYTicks(dataMin, dataMax);
    const yScale = yTicksInfo.yScale;
    return {
      max: yTicksInfo.max ? yTicksInfo.max : dataMax,
      min: dataMin,
      ticks: yTicksInfo.ticks,
      yScale
    };
  }

  get xScaling() {
    const { data, dimensions, sortedTimeRange, isMultiRange, timeInterval, showEveryTick } = this.args;
    if(!data || !dimensions) return [0,0]
    const xScale = isMultiRange
      ? sortedTimeRange.map(timeRange => this.makeXScale(timeRange))
      : this.xScaleRange.domain(sortedTimeRange);
    
    const longestRange = this.getLongestXRangeTime(sortedTimeRange);
    const range = isMultiRange ? sortedTimeRange[0] : sortedTimeRange;
    const startTime = range[0].getTime();
    const endTime = startTime + longestRange;
    const fixedInterval = !timeInterval || timeInterval < 900000 ? 900000 : timeInterval; // cap at 15 min
    const idealTickCount = showEveryTick ? ((endTime - startTime) / fixedInterval) + 1 : 2; // tick for every interval + origin
    const labelCount = this.xLabelCount(); // number of labels based on dimensions of chart
    const tickDensity = idealTickCount / dimensions.width * 100; // the lower the value, the more spread apart they are
    const tooDense = tickDensity < 6;
    const tickCount = tooDense ? idealTickCount : idealTickCount / 2;
    const labelStep = Math.ceil(tickCount / labelCount);
    return {
      xScale,
      ticks: Array.from({ length: tickCount }, (_, i) => {
        const time = showEveryTick ? i * (tooDense ? fixedInterval : fixedInterval * 2) + startTime : (i * longestRange) + startTime;
        const value = new Date(time);
        const x = isMultiRange ? xScale[0](value) : xScale(value);
        const lastTick = tickCount - 1 === i;
        return {
          label: i % labelStep || i > (7 / 8) * tickCount ? "" : this.formatDate(value, fixedInterval >= 1000 * 60 * 60 * 24),
          x,
          position: `left: ${x}px;`,
          shiftLabelBack: lastTick && (dimensions.width - 100 < x) // If the final label is going to be beyond the svg chart edge, add class to translate -100%
        }
      })
    };
  }

  // This returns the timespan of the longest timerange. This is used to calculate where the end of the x axis is
  getLongestXRangeTime(range) {
    const longestRange = this.args.isMultiRange
      ? Math.abs(range[0][1] - range[0][0]) > Math.abs(range[0][1] - range[0][0]) ? range[0] : range[1]
      : range;
    
    return Math.abs(longestRange[1] - longestRange[0]);
  }

  makeXScale(range) {
    return this.xScaleRange.domain(range);
  }

  xLabelCount() {
    const width = this.args.dimensions.width;
    if (!this.args.showEveryTick) return 2;
    if(width < 300) {
      return 3;
    } else if (width < 600) {
      return 4;
    } else {
      return 6;
    }
  }

  getYMax() {
    const { yMax, data, scaleYBasedOnTotalDataset, filteredData, isMultiDataset, isMultiRange, yKey } = this.args;
    if(yMax) { // override with yMax argument for fixed max
      return yMax
    } else if(data) {
      const maxData = scaleYBasedOnTotalDataset ? data : filteredData;
      const dataset = isMultiDataset || (!scaleYBasedOnTotalDataset && isMultiRange) ? maxData.flat(1) : maxData;
      return dataset.length > 0 ? Math.max(...dataset.map(d => d[yKey])) : 0;
    } else {
      return 0;
    }
  }

  getYMin() {
    const { y0, yKey, data, filteredData, isMultiDataset, isMultiRange, scaleYBasedOnTotalDataset, autoY0 } = this.args;
    if(y0) return y0;
    const minData = scaleYBasedOnTotalDataset ? data : filteredData;
    const dataset = isMultiDataset || (!scaleYBasedOnTotalDataset && isMultiRange) ? minData.flat(1) : minData
    return autoY0 && dataset.length > 0 ? Math.min(...dataset.map(d => d[yKey])) : y0;
  }

  pad(number, padding = -2) {
    return ('0' + number).slice(padding)
  }
  
  formatDate(date) {
    return format(date, this.dateLabelFormat);
  }
  
  tickStepSize(range, targetSteps) {
    const startingStep = range / targetSteps;
     // magnitude
    const magnitude = Math.floor(Math.log(startingStep) / Math.LN10);
    const MagnitudePower = Math.pow(10, magnitude);
     // signif
    let residual = Math.round(startingStep/MagnitudePower);
    if( residual >= 5.0 ) {
      residual = 10.0;
    } else if ( residual >= 2.0 ) {
      residual = 5.0;
    } else if ( residual >= 1.0 ) {
      residual = 2.0;
    }

    return residual * MagnitudePower;
  }
}

==> sli-plots-0.0.33.tgz: package/addon/components/plot-axes-and-ticks.js <==
import Component from '@glimmer/component';
import { scaleTime, scaleLinear } from "d3-scale";
import { format } from 'date-fns'

export default class PlotAxesAndTicksComponent extends Component {
  get displayingXTicksOrLabels() {
    return this.args.showXTicks || !this.args.hideXTickLabels;
  }

  get dateLabelFormat() {
    return this.args.dateLabelFormat ?? this.smartDateLabel();
  }

  smartDateLabel() {
    if (!this.args.sortedTimeRange) return "EEE";
    const timespan = this.getLongestXRangeTime(this.args.sortedTimeRange);
    if (timespan <= 1000 * 60 * 60 * 24 * 8) { // daily
      return "EEE";
    } else if (timespan <= 1000 * 60 * 60 * 24 * 31) { // Monthly
      return "MMM d";
    } else { // Yearly
      return "yyyy";
    }
  }

  get xScaleRange() {
    return this.args.dimensions
    ? scaleTime().range([0, this.args.dimensions.width])
    : {}
  }

  get yScaleRange() {
    return this.args.dimensions
    ? scaleLinear().range([this.args.dimensions.height, 0])
      : {}
  }

  makeYTicks(originalMin, max) {
    const { isMultiDataset, data, hasData, dimensions, showEveryTick } = this.args;
    const min = this.roundMinimum(originalMin, max);
    if((isMultiDataset && data[0]?.length === 0) || !hasData) {
      return { yScale: this.yScaleRange.domain ? this.yScaleRange.domain([min, max]) : () => {} }
    }

    const tickCount = dimensions.height < 500 ? 7 : 10;
    const stepSize = this.tickStepSize(max - min, tickCount);
    const maxTick = Math.ceil(max / stepSize) * stepSize;
    const yScale = this.yScaleRange.domain([min, maxTick]);
    return {
      max: maxTick,
      min,
      yScale,
      ticks: Array.from({length: showEveryTick ? 2 : Math.ceil((maxTick - min)/(stepSize + 1))}, (_, i) => {
        const value = showEveryTick
                        ? i === 0 ? min : maxTick
                        : i * stepSize + min
        const fixedValue = value < 1 && value !== 0 ? value.toFixed(2) : value;
        const y = yScale(fixedValue);
        return {
          label: `${fixedValue}`,
          fixedValue,
          y,
          position: `top: ${y}px;`
        }
      })
    }
  }

  // This method returns a new minimum value that is rounded down based on
  // the magnatude of of min max for a nice rounded value that isn't too far off
  // from the lowest value
  roundMinimum(min, max) {
    if(min === 0) return min;

    const diff = max - min;

    if(diff < 20) return Math.floor(min);
    if(diff < 100) return Math.floor(min / 10) * 10;
    if(diff < 2000) return Math.floor(min / 100) * 100;

    return Math.floor(min / 1000) * 1000;
  }

  get yScaling() {
    if(!this.args.data) return [0,0]
    const dataMax = this.getYMax();
    const dataMin = this.getYMin();
    const yTicksInfo = this.makeYTicks(dataMin, dataMax);
    const yScale = yTicksInfo.yScale;
    return {
      max: yTicksInfo.max ? yTicksInfo.max : dataMax,
      min: dataMin,
      ticks: yTicksInfo.ticks,
      yScale
    };
  }

  get xScaling() {
    const { data, dimensions, sortedTimeRange, isMultiRange, timeInterval, showEveryTick } = this.args;
    if(!data || !dimensions) return [0,0]
    const xScale = isMultiRange
      ? sortedTimeRange.map(timeRange => this.makeXScale(timeRange))
      : this.xScaleRange.domain(sortedTimeRange);
    
    const longestRange = this.getLongestXRangeTime(sortedTimeRange);
    const range = isMultiRange ? sortedTimeRange[0] : sortedTimeRange;
    const startTime = range[0].getTime();
    const endTime = startTime + longestRange;
    const fixedInterval = !timeInterval || timeInterval < 900000 ? 900000 : timeInterval; // cap at 15 min
    const idealTickCount = showEveryTick ? ((endTime - startTime) / fixedInterval) + 1 : 2; // tick for every interval + origin
    const labelCount = this.xLabelCount(); // number of labels based on dimensions of chart
    const tickDensity = idealTickCount / dimensions.width * 100; // the lower the value, the more spread apart they are
    const tooDense = tickDensity < 6;
    const tickCount = tooDense ? idealTickCount : idealTickCount / 2;
    const labelStep = Math.ceil(tickCount / labelCount);
    return {
      xScale,
      ticks: Array.from({ length: tickCount }, (_, i) => {
        const time = showEveryTick ? i * (tooDense ? fixedInterval : fixedInterval * 2) + startTime : (i * longestRange) + startTime;
        const value = new Date(time);
        const x = isMultiRange ? xScale[0](value) : xScale(value);
        const lastTick = tickCount - 1 === i;
        return {
          label: i % labelStep || i > (7 / 8) * tickCount ? "" : this.formatDate(value, fixedInterval >= 1000 * 60 * 60 * 24),
          x,
          position: `left: ${x}px;`,
          shiftLabelBack: lastTick && (dimensions.width - 100 < x) // If the final label is going to be beyond the svg chart edge, add class to translate -100%
        }
      })
    };
  }

  // This returns the timespan of the longest timerange. This is used to calculate where the end of the x axis is
  getLongestXRangeTime(range) {
    const longestRange = this.args.isMultiRange
      ? Math.abs(range[0][1] - range[0][0]) > Math.abs(range[0][1] - range[0][0]) ? range[0] : range[1]
      : range;
    
    return Math.abs(longestRange[1] - longestRange[0]);
  }

  makeXScale(range) {
    return this.xScaleRange.domain(range);
  }

  xLabelCount() {
    const width = this.args.dimensions.width;
    if (!this.args.showEveryTick) return 2;
    if(width < 300) {
      return 3;
    } else if (width < 600) {
      return 4;
    } else {
      return 6;
    }
  }

  getYMax() {
    const { yMax, data, scaleYBasedOnTotalDataset, filteredData, isMultiDataset, isMultiRange, yKey } = this.args;
    if(yMax) { // override with yMax argument for fixed max
      return yMax
    } else if(data) {
      const maxData = scaleYBasedOnTotalDataset ? data : filteredData;
      const dataset = isMultiDataset || (!scaleYBasedOnTotalDataset && isMultiRange) ? maxData.flat(1) : maxData;
      return dataset?.length > 0 ? Math.max(...dataset.map(d => d[yKey])) : 0;
    } else {
      return 0;
    }
  }

  getYMin() {
    const { y0, yKey, data, filteredData, isMultiDataset, isMultiRange, scaleYBasedOnTotalDataset, autoY0 } = this.args;
    if(y0) return y0;
    const minData = scaleYBasedOnTotalDataset ? data : filteredData;
    const dataset = isMultiDataset || (!scaleYBasedOnTotalDataset && isMultiRange) ? minData.flat(1) : minData
    return autoY0 && dataset?.length > 0 ? Math.min(...dataset.map(d => d[yKey])) : y0;
  }

  pad(number, padding = -2) {
    return ('0' + number).slice(padding)
  }
  
  formatDate(date) {
    return format(date, this.dateLabelFormat);
  }
  
  tickStepSize(range, targetSteps) {
    const startingStep = range / targetSteps;
     // magnitude
    const magnitude = Math.floor(Math.log(startingStep) / Math.LN10);
    const MagnitudePower = Math.pow(10, magnitude);
     // signif
    let residual = Math.round(startingStep/MagnitudePower);
    if( residual >= 5.0 ) {
      residual = 10.0;
    } else if ( residual >= 2.0 ) {
      residual = 5.0;
    } else if ( residual >= 1.0 ) {
      residual = 2.0;
    }

    return residual * MagnitudePower;
  }
}
